/// Josh's User Interface Library
/// A C++20 Library for creating, styling, and rendering of a UI/UX widgets.
/// Developed and Maintained by Josh O'Leary @ Redacted Software.
/// Special Thanks to William Tomasine II and Maxine Hayes.
/// (c) 2024 Redacted Software
/// This work is dedicated to the public domain.

/// @file ScrollingRect.hpp
/// @desc Scrolling Rectangle Widget
/// @edit 2024-10-11

#pragma once
#include <JUI/Widgets/Rect.hpp>
namespace JUI {
    class ScrollingRect;
}

// TODO: Fix direction of ScrollBar itself
// TODO: MouseWheel scroll, and ScrollBar clickable
// TODO: Clamp range of ScrollBar


/// A Rectangle Widget which has a larger renderable area than the visible area.
/// This allows user-controlled scrolling of the viewable content.
class JUI::ScrollingRect : public Rect {
protected:
    bool vertical_scrollbar_enabled = true;
    bool horizontal_scrollbar_enabled = true;
    bool scrollbar_visible = true;
    float scrollbar_width = 12;
    Color4 scrollbar_color = Colors::Whites::Azure;
    float scroll = 0;
    JGL::RenderTarget* canvas = nullptr;

protected:
    /* This isn't public because nothing should ever
     * have to do this from the outside. -Redacted */
    void RecomputeRenderTarget();
public:
    float scroll_size = 0;
    JGL::RenderTarget* GetCanvas();
    [[nodiscard]] Vector2i CanvasSize() const;
    [[nodiscard]] float ScrollPos() const { return scroll; }
    [[nodiscard]] Vector2 CanvasPosition() const;
    // TODO scrolling in either direction. Assuming vertical scroll for now.
    Vector2 CanvasAbsolutePosition() const;
    Vector2 CanvasAbsoluteSize() const;
public:
    void ScrollPos(float pos) { scroll = pos; }
    void CanvasSize(const Vector2i& new_size);
    void InnerDraw() override;
    void Draw() override;
    void Update(float delta) override {
        //scroll += delta*5;
        //canvas->Resize(Vector2i(GetAbsoluteSize().x, GetAbsoluteSize().y));
        Rect::Update(delta);
    }

    void ObserveKeyInput(Key key, bool pressed) override
    {
        if (key == Keys::UpArrow && pressed)
        {
            scroll -= 10;
        }
        if (key == Keys::DownArrow && pressed)
        {
            scroll += 10;
        }

    }
public:
    ~ScrollingRect() override;

    ScrollingRect();
    explicit ScrollingRect(Widget* parent);;
};